@@ -5,7 +5,7 @@ use Carp;
 use DBI;
 use base qw(DBI::db);
 
-our $VERSION = '0.11';
+our $VERSION = '0.12';
 
 =head1 NAME
 
@@ -112,13 +112,13 @@ Specify how the results should be ordered.  This option can take various values:
 
 =item * a straight scalar or arrayref sorts by the given column(s):
 
-    { order_by => 'foo' }         # equivalent to "ORDER BY foo"
-    { order_by => [ qw(foo bar) } # equiv to "ORDER BY foo,bar"
+    { order_by => 'foo' }           # equivalent to "ORDER BY foo"
+    { order_by => [ qw(foo bar) ] } # equiv to "ORDER BY foo,bar"
 
 =item * a hashref of C<order => column name>, e.g.:
 
     { order_by => { desc => 'foo' } } # equiv to ORDER BY foo DESC
-    { order_by => [ { desc => 'foo' }, { asc => 'bar' }
+    { order_by => [ { desc => 'foo' }, { asc => 'bar' } ] }
        # above is equiv to ORDER BY foo ASC, bar DESC
 
 =back
@@ -134,7 +134,7 @@ query anyway, so you needn't add it yourself.
 An example of using options to control the results you get back:
 
     # Get the name & phone number of the 10 highest-paid men:
-    database->query(
+    database->quick_select(
         'employees', 
         { gender => 'male' },
         { order_by => 'salary', limit => 10, columns => [qw(name phone)] }
@@ -287,7 +287,7 @@ sub _quick_query {
 
     # Add an ORDER BY clause, if we want to:
     if (exists $opts->{order_by}) {
-        $sql .= $self->_build_order_by_clause($opts->{order_by});
+        $sql .= ' ' . $self->_build_order_by_clause($opts->{order_by});
     }
 
 
